@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000900
+#define IP_VS_VERSION_CODE            0x000901
 #define NVERSION(version)                       \
 	(version >> 16) & 0xFF,                 \
 	(version >> 8) & 0xFF,                  \
@@ -112,8 +112,6 @@ struct ip_vs_rule_user {
 #include <net/dst.h>			/* for struct dst_entry */
 
 
-/*  #define CONFIG_IP_VS_DEBUG */
-
 #ifdef CONFIG_IP_VS_DEBUG
 extern int ip_vs_get_debug_level(void);
 #define IP_VS_DBG(level, msg...)                        \
@@ -184,6 +182,7 @@ enum {
 	NET_IPV4_VS_LBLCR_EXPIRE=20,
 	NET_IPV4_VS_USE_DST=21,
 	NET_IPV4_VS_CACHE_BYPASS=22,
+	NET_IPV4_VS_EXPIRE_NODEST_CONN=23,
 };
 
 
@@ -548,6 +547,7 @@ extern struct ip_vs_scheduler *ip_vs_lookup_scheduler(const char *sched_name);
  */
 extern int sysctl_use_dst;
 extern int sysctl_ip_vs_cache_bypass;
+extern int sysctl_ip_vs_expire_nodest_conn;
 extern atomic_t ip_vs_dropentry;
 extern struct ip_vs_stats ip_vs_stats;
 extern rwlock_t __ip_vs_svc_lock;
