@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.16 2001/05/09 15:01:15 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.17 2001/06/04 13:51:03 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -55,8 +55,11 @@ static struct list_head *ip_vs_conn_tab;
 /*  SLAB cache for IPVS connections */
 static kmem_cache_t *ip_vs_conn_cachep;
 
+/*  counter for current IPVS connections */
+static atomic_t ip_vs_conn_count = ATOMIC_INIT(0);
+
 /*
- *  No client port connection couter
+ *  No client port connection counter
  */
 static atomic_t ip_vs_conn_no_cport_cnt = ATOMIC_INIT(0);
 
@@ -1228,9 +1231,9 @@ static void ip_vs_conn_expire(unsigned long data)
 		ip_vs_timeout_detach(cp);
 		if (cp->flags & IP_VS_CONN_F_NO_CPORT)
 			atomic_dec(&ip_vs_conn_no_cport_cnt);
+		atomic_dec(&ip_vs_conn_count);
+
 		kmem_cache_free(ip_vs_conn_cachep, cp);
-		/*  sysctl_ip_always_defrag--; */
-		MOD_DEC_USE_COUNT;
 		goto expire_out;
 	}
 
@@ -1261,7 +1264,6 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 			IP_VS_ERR("ip_vs_conn_new: no memory available.\n");
 		return NULL;
 	}
-	MOD_INC_USE_COUNT;
 
 	memset(cp, 0, sizeof(*cp));
 	INIT_LIST_HEAD(&cp->c_list);
@@ -1284,6 +1286,7 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
 	atomic_set(&cp->n_control, 0);
 	atomic_set(&cp->in_pkts, 0);
 
+	atomic_inc(&ip_vs_conn_count);
 	if (flags & IP_VS_CONN_F_NO_CPORT)
 		atomic_inc(&ip_vs_conn_no_cport_cnt);
 
@@ -1463,6 +1466,53 @@ void ip_vs_random_dropentry(void)
 }
 
 
+/*
+ *      Flush all the connection entries in the ip_vs_conn_tab
+ */
+static void ip_vs_conn_flush(void)
+{
+	int idx;
+	struct ip_vs_conn *cp;
+	struct list_head *l,*e;
+	struct ip_vs_conn *ct;
+	void (*fn)(unsigned long);
+
+ flush_again:
+	for (idx=0; idx<IP_VS_CONN_TAB_SIZE; idx++) {
+		/*
+		 *  Lock is actually needed in this loop.
+		 */
+		ct_write_lock(idx);
+
+		l = &ip_vs_conn_tab[idx];
+		for (e=l->next; e!=l; e=e->next) {
+			cp = list_entry(e, struct ip_vs_conn, c_list);
+			ct_write_unlock(idx);
+			IP_VS_DBG(4, "del connection\n");
+			fn = (cp->timer).function;
+			if (del_sltimer(&cp->timer)) {
+				ct = cp->control;
+				fn((unsigned long)cp);
+				if (ct && !atomic_read(&ct->n_control)) {
+					IP_VS_DBG(4, "del conn template\n");
+					if(del_sltimer(&ct->timer))
+						fn((unsigned long)ct);
+				}
+			}
+			ct_write_lock(idx);
+		}
+		ct_write_unlock(idx);
+	}
+
+	/* the counter may be not NULL, because maybe some conn entries
+	   are run by slow timer handler or unhashed but still referred */
+	if (atomic_read(&ip_vs_conn_count) != 0) {
+		schedule();
+		goto flush_again;
+	}
+}
+
+
 int ip_vs_conn_init(void)
 {
 	int idx;
@@ -1506,6 +1556,9 @@ int ip_vs_conn_init(void)
 
 void ip_vs_conn_cleanup(void)
 {
+	/* flush all the connection entries first */
+	ip_vs_conn_flush();
+
 	/* Release the empty cache */
 	kmem_cache_destroy(ip_vs_conn_cachep);
 	proc_net_remove("ip_vs_conn");
