@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.18 2001/05/09 15:01:15 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.19 2001/06/04 13:51:03 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1109,15 +1109,27 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 			       iph->daddr, h.portp[1]);
 
 	/*
-	 * Checking the server status
+	 * Check the server status
 	 */
 	if (cp && cp->dest && !(cp->dest->flags & IP_VS_DEST_F_AVAILABLE)) {
-		/*
-		 * If the dest is not avaiable, start the timer of the conn
-		 * with its original expires, and silently drop the packet.
-		 */
-		__ip_vs_conn_put(cp);
-		return NF_DROP;
+		/* the destination server is not availabe */
+
+		if (sysctl_ip_vs_expire_nodest_conn) {
+			/* expire the connection immediately */
+			__ip_vs_conn_put(cp);
+			if (del_sltimer(&cp->timer)) {
+				void (*fn)(unsigned long)
+					= (cp->timer).function;
+				fn((unsigned long)cp);
+			}
+
+			return NF_DROP;
+		} else {
+			/* don't restart its timer, and silently
+			   drop the packet. */
+			__ip_vs_conn_put(cp);
+			return NF_DROP;
+		}
 	}
 
 	if (!cp
