@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.16 2001/05/09 15:01:15 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.17 2001/06/04 13:51:03 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -75,6 +75,7 @@ static int sysctl_ip_vs_amemthresh = 1024;
 static int sysctl_ip_vs_am_droprate = 10;
 int sysctl_use_dst = 1;
 int sysctl_ip_vs_cache_bypass = 0;
+int sysctl_ip_vs_expire_nodest_conn = 0;
 
 #ifdef CONFIG_IP_VS_DEBUG
 static int sysctl_ip_vs_debug_level = 0;
@@ -1289,7 +1290,7 @@ static int ip_vs_sysctl_defense_mode(ctl_table *ctl, int write,
  */
 struct ip_vs_sysctl_table {
 	struct ctl_table_header *sysctl_header;
-	ctl_table vs_vars[21];
+	ctl_table vs_vars[22];
 	ctl_table vs_dir[2];
 	ctl_table ipv4_dir[2];
 	ctl_table root_dir[2];
@@ -1360,6 +1361,9 @@ static struct ip_vs_sysctl_table ipv4_vs_table = {
 	 {NET_IPV4_VS_CACHE_BYPASS, "cache_bypass",
 	  &sysctl_ip_vs_cache_bypass, sizeof(int), 0644, NULL,
 	  &proc_dointvec},
+	 {NET_IPV4_VS_EXPIRE_NODEST_CONN, "expire_nodest_conn",
+	  &sysctl_ip_vs_expire_nodest_conn, sizeof(int), 0644, NULL,
+	  &proc_dointvec},
 	 {0}},
 	{{NET_IPV4_VS, "vs", NULL, 0, 0555, ipv4_vs_table.vs_vars},
 	 {0}},
