@@ -1,10 +1,10 @@
 /*
  * IP_VS        ftp application module
  *
- * Version:	$Id: ip_vs_ftp.c,v 1.5 2001/03/18 15:01:45 wensong Exp $
+ * Version:	$Id: ip_vs_ftp.c,v 1.6 2001/05/09 15:01:15 wensong Exp $
  *
  * Authors:	Wensong Zhang <wensong@linuxvirtualserver.org>
- *		
+ *
  * Changes:
  *
  *
@@ -12,7 +12,7 @@
  *	modify it under the terms of the GNU General Public License
  *	as published by the Free Software Foundation; either version
  *	2 of the License, or (at your option) any later version.
- *	
+ *
  * Most code here is taken from ip_masq_ftp.c in kernel 2.2. The difference
  * is that ip_vs_ftp module handles the reverse direction to ip_masq_ftp.
  *
@@ -21,7 +21,7 @@
  * Version:	@(#)ip_masq_ftp.c 0.04   02/05/96
  *
  * Author:	Wouter Gadeyne
- *		
+ *
  */
 
 #include <linux/config.h>
@@ -43,7 +43,7 @@
 #define CLIENT_STRING "PORT "
 
 
-/* 
+/*
  * List of ports (up to IP_VS_APP_MAX_PORTS) to be handled by helper
  * First port is set to the default port.
  */
@@ -67,14 +67,14 @@ static int ip_vs_ftp_pasv;
 static int
 ip_vs_ftp_init_conn(struct ip_vs_app *mapp, struct ip_vs_conn *cp)
 {
-        return 0;
+	return 0;
 }
 
 
 static int
 ip_vs_ftp_done_conn(struct ip_vs_app *mapp, struct ip_vs_conn *cp)
 {
-        return 0;
+	return 0;
 }
 
 
@@ -95,8 +95,8 @@ static int ip_vs_ftp_get_addrport(char *data, char *data_limit,
 			data++;
 			continue;
 		}
-                *start = data+plen;
- 		p1 = simple_strtoul(data+plen, &data, 10);
+		*start = data+plen;
+		p1 = simple_strtoul(data+plen, &data, 10);
 		if (*data != ',')
 			continue;
 		p2 = simple_strtoul(data+1, &data, 10);
@@ -114,13 +114,13 @@ static int ip_vs_ftp_get_addrport(char *data, char *data_limit,
 		p6 = simple_strtoul(data+1, &data, 10);
 		if (*data != term)
 			continue;
-                
-                *end = data;
+
+		*end = data;
 		*addr = (p4<<24) | (p3<<16) | (p2<<8) | p1;
 		*port = (p6<<8) | p5;
-                return 1;
-        }
-        return 0;
+		return 1;
+	}
+	return 0;
 }
 
 
@@ -140,61 +140,61 @@ static int ip_vs_ftp_get_addrport(char *data, char *data_limit,
 static int ip_vs_ftp_out(struct ip_vs_app *mapp,
 			 struct ip_vs_conn *cp, struct sk_buff **skb_p)
 {
-        struct sk_buff *skb;
+	struct sk_buff *skb;
 	struct iphdr *iph;
 	struct tcphdr *th;
 	char *data, *data_limit;
-        char *start, *end;
+	char *start, *end;
 	__u32 from;
 	__u16 port;
 	struct ip_vs_conn *n_cp;
 	char buf[24];		/* xxx.xxx.xxx.xxx,ppp,ppp\000 */
-        unsigned buf_len;
+	unsigned buf_len;
 	int diff;
 
 	/* Only useful for established sessions */
-	if (cp->state != IP_VS_S_ESTABLISHED) 
+	if (cp->state != IP_VS_S_ESTABLISHED)
 		return 0;
 
 	if (cp->app_data == &ip_vs_ftp_pasv) {
-                skb = *skb_p;
-                iph = skb->nh.iph;
-                th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
-                data = (char *)th + (th->doff << 2);
-                data_limit = skb->h.raw + skb->len;
+		skb = *skb_p;
+		iph = skb->nh.iph;
+		th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
+		data = (char *)th + (th->doff << 2);
+		data_limit = skb->h.raw + skb->len;
 
-                if (ip_vs_ftp_get_addrport(data, data_limit,
-                                           SERVER_STRING,
-                                           sizeof(SERVER_STRING)-1, ')',
-                                           &from, &port,
-                                           &start, &end) == 0)
-                        return 0;
-        
-                IP_VS_DBG(1-debug, "PASV response (%u.%u.%u.%u:%d) -> "
-                          "%u.%u.%u.%u:%d detected\n",
-                          NIPQUAD(from), ntohs(port), NIPQUAD(cp->caddr), 0);
+		if (ip_vs_ftp_get_addrport(data, data_limit,
+					   SERVER_STRING,
+					   sizeof(SERVER_STRING)-1, ')',
+					   &from, &port,
+					   &start, &end) == 0)
+			return 0;
 
-                /*
-                 * Now update or create an connection entry for it
-                 */
-                n_cp = ip_vs_conn_out_get(iph->protocol, from, port,
-                                          cp->caddr, 0);
-                if (!n_cp) {
-                        n_cp = ip_vs_conn_new(IPPROTO_TCP,
-                                              cp->caddr, 0,
-                                              cp->vaddr, port,
-                                              from, port,
-                                              IP_VS_CONN_F_NO_CPORT);
-                        if (!n_cp)
-                                return 0;
+		IP_VS_DBG(1-debug, "PASV response (%u.%u.%u.%u:%d) -> "
+			  "%u.%u.%u.%u:%d detected\n",
+			  NIPQUAD(from), ntohs(port), NIPQUAD(cp->caddr), 0);
 
-                        /* add its controller and bind its destination */
-                        ip_vs_control_add(n_cp, cp);
-                        ip_vs_bind_dest(n_cp, cp->dest);
-                        
-                        /* increase dest's inactive connection counter */
-                        atomic_inc(&cp->dest->inactconns);
-                }
+		/*
+		 * Now update or create an connection entry for it
+		 */
+		n_cp = ip_vs_conn_out_get(iph->protocol, from, port,
+					  cp->caddr, 0);
+		if (!n_cp) {
+			n_cp = ip_vs_conn_new(IPPROTO_TCP,
+					      cp->caddr, 0,
+					      cp->vaddr, port,
+					      from, port,
+					      IP_VS_CONN_F_NO_CPORT,
+					      cp->dest);
+			if (!n_cp)
+				return 0;
+
+			/* add its controller */
+			ip_vs_control_add(n_cp, cp);
+
+			/* increase dest's inactive connection counter */
+			atomic_inc(&cp->dest->inactconns);
+		}
 
 		/*
 		 * Replace the old passive address with the new one
@@ -206,31 +206,31 @@ static int ip_vs_ftp_out(struct ip_vs_app *mapp,
 		buf_len = strlen(buf);
 
 		IP_VS_DBG(1-debug, "new PORT %u.%u.%u.%u:%d\n",
-                          NIPQUAD(from), ntohs(port));
+			  NIPQUAD(from), ntohs(port));
 
 		/*
 		 * Calculate required delta-offset to keep TCP happy
 		 */
 		diff = buf_len - (end-start);
-		
+
 		/*
 		 *	No shift.
 		 */
 		if (diff==0) {
 			/*
 			 * simple case, just replace the old PORT cmd
- 			 */
- 			memcpy(start, buf, buf_len);
- 		} else {
+			 */
+			memcpy(start, buf, buf_len);
+		} else {
 			*skb_p = ip_vs_skb_replace(skb, GFP_ATOMIC, start,
-                                                   end-start, buf, buf_len);
+						   end-start, buf, buf_len);
 		}
 
 		cp->app_data = NULL;
 		ip_vs_conn_put(n_cp);
 
-                return diff;
-        }
+		return diff;
+	}
 	return 0;
 }
 
@@ -259,33 +259,33 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
 	struct ip_vs_conn *n_cp;
 
 	/* Only useful for established sessions */
-	if (cp->state != IP_VS_S_ESTABLISHED) 
+	if (cp->state != IP_VS_S_ESTABLISHED)
 		return 0;
 
 	/*
 	 * Detecting whether it is passive
 	 */
-        skb = *skb_p;
+	skb = *skb_p;
 	iph = skb->nh.iph;
-        th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
+	th = (struct tcphdr *)&(((char *)iph)[iph->ihl*4]);
 
-        /* Since there may be OPTIONS in the TCP packet and the HLEN is
-           the length of the header in 32-bit multiples, it is accurate
-           to caculate data address by th+HLEN*4 */
-        data = data_start = (char *)th + (th->doff << 2);
-        data_limit = skb->h.raw + skb->len;
+	/* Since there may be OPTIONS in the TCP packet and the HLEN is
+	   the length of the header in 32-bit multiples, it is accurate
+	   to caculate data address by th+HLEN*4 */
+	data = data_start = (char *)th + (th->doff << 2);
+	data_limit = skb->h.raw + skb->len;
 
 	while (data < data_limit) {
-                if (strnicmp(data, "PASV\r\n", 6) == 0) {
-                        IP_VS_DBG(1-debug, "got PASV at %d of %d\n",
-                                  data - data_start,
-                                  data_limit - data_start);
-                        cp->app_data = &ip_vs_ftp_pasv;
-                        return 0;
-                }
+		if (strnicmp(data, "PASV\r\n", 6) == 0) {
+			IP_VS_DBG(1-debug, "got PASV at %d of %d\n",
+				  data - data_start,
+				  data_limit - data_start);
+			cp->app_data = &ip_vs_ftp_pasv;
+			return 0;
+		}
 		data++;
-        }
-        
+	}
+
 	/*
 	 * To support virtual FTP server, the scenerio is as follows:
 	 *       FTP client ----> Load Balancer ----> FTP server
@@ -293,70 +293,70 @@ static int ip_vs_ftp_in(struct ip_vs_app *mapp,
 	 * then create a new masquerading entry for the coming data
 	 * connection.
 	 */
-        data = data_start;
-        data_limit = skb->h.raw + skb->len - 18;
+	data = data_start;
+	data_limit = skb->h.raw + skb->len - 18;
 
-        if (ip_vs_ftp_get_addrport(data, data_limit,
-                                   CLIENT_STRING, sizeof(CLIENT_STRING)-1,
-                                   '\r', &to, &port,
-                                   &start, &end) == 0)
-                return 0;
-        
-        IP_VS_DBG(1-debug, "PORT %u.%u.%u.%u:%d detected\n",
-                  NIPQUAD(to), ntohs(port));
+	if (ip_vs_ftp_get_addrport(data, data_limit,
+				   CLIENT_STRING, sizeof(CLIENT_STRING)-1,
+				   '\r', &to, &port,
+				   &start, &end) == 0)
+		return 0;
 
-        /*
-         * Now update or create an masquerade entry for it
-         */
-        IP_VS_DBG(1-debug, "protocol %s %u.%u.%u.%u:%d %u.%u.%u.%u:%d\n",
-                  ip_vs_proto_name(iph->protocol),
-                  NIPQUAD(to), ntohs(port), NIPQUAD(iph->daddr), 0);
+	IP_VS_DBG(1-debug, "PORT %u.%u.%u.%u:%d detected\n",
+		  NIPQUAD(to), ntohs(port));
 
-        n_cp = ip_vs_conn_in_get(iph->protocol,
-                                 to, port,
-                                 iph->daddr, htons(ntohs(cp->vport)-1));
-        if (!n_cp) {
-                n_cp = ip_vs_conn_new(IPPROTO_TCP,
-                                      to, port,
-                                      cp->vaddr, htons(ntohs(cp->vport)-1),
-                                      cp->daddr, htons(ntohs(cp->dport)-1),
-                                      0);
-                if (!n_cp)
-                        return 0;
-                
-                /* add its controller and bind its destination */
-                ip_vs_control_add(n_cp, cp);
-                ip_vs_bind_dest(n_cp, cp->dest);
+	/*
+	 * Now update or create an masquerade entry for it
+	 */
+	IP_VS_DBG(1-debug, "protocol %s %u.%u.%u.%u:%d %u.%u.%u.%u:%d\n",
+		  ip_vs_proto_name(iph->protocol),
+		  NIPQUAD(to), ntohs(port), NIPQUAD(iph->daddr), 0);
 
-                /* increase dest's inactive connection counter */
-                atomic_inc(&cp->dest->inactconns);
-        }
+	n_cp = ip_vs_conn_in_get(iph->protocol,
+				 to, port,
+				 iph->daddr, htons(ntohs(cp->vport)-1));
+	if (!n_cp) {
+		n_cp = ip_vs_conn_new(IPPROTO_TCP,
+				      to, port,
+				      cp->vaddr, htons(ntohs(cp->vport)-1),
+				      cp->daddr, htons(ntohs(cp->dport)-1),
+				      0,
+				      cp->dest);
+		if (!n_cp)
+			return 0;
 
-        /*
-         * 	Move tunnel to listen state
-         */
-        /*  ip_vs_conn_listen(n_cp); */
-        ip_vs_conn_put(n_cp);
+		/* add its controller */
+		ip_vs_control_add(n_cp, cp);
+
+		/* increase dest's inactive connection counter */
+		atomic_inc(&cp->dest->inactconns);
+	}
+
+	/*
+	 *	Move tunnel to listen state
+	 */
+	/*  ip_vs_conn_listen(n_cp); */
+	ip_vs_conn_put(n_cp);
 
 	/* no diff required for incoming packets, thank goodness */
-        return 0;
+	return 0;
 }
 
 
 static struct ip_vs_app ip_vs_ftp = {
-        {0},			/* n_list */
+	{0},			/* n_list */
 	"ftp",			/* name */
-        0,                      /* type */
-        THIS_MODULE,            /* this module */
-        ip_vs_ftp_init_conn,    /* ip_vs_init_conn */
-        ip_vs_ftp_done_conn,    /* ip_vs_done_conn */
-        ip_vs_ftp_out,          /* pkt_out */
-        ip_vs_ftp_in,           /* pkt_in */
+	0,                      /* type */
+	THIS_MODULE,            /* this module */
+	ip_vs_ftp_init_conn,    /* ip_vs_init_conn */
+	ip_vs_ftp_done_conn,    /* ip_vs_done_conn */
+	ip_vs_ftp_out,          /* pkt_out */
+	ip_vs_ftp_in,           /* pkt_in */
 };
 
 
 /*
- * 	ip_vs_ftp initialization
+ *	ip_vs_ftp initialization
  */
 static int __init ip_vs_ftp_init(void)
 {
@@ -365,22 +365,22 @@ static int __init ip_vs_ftp_init(void)
 	for (i=0; i<IP_VS_APP_MAX_PORTS; i++) {
 		if (ports[i]) {
 			if (!(incarnations[i] =
-                             kmalloc(sizeof(struct ip_vs_app), GFP_KERNEL)))
+			     kmalloc(sizeof(struct ip_vs_app), GFP_KERNEL)))
 				return -ENOMEM;
-                        
+
 			memcpy(incarnations[i], &ip_vs_ftp,
-                               sizeof(struct ip_vs_app));
-			if ((j = register_ip_vs_app(incarnations[i], 
-                                                    IPPROTO_TCP, 
-                                                    ports[i]))) {
+			       sizeof(struct ip_vs_app));
+			if ((j = register_ip_vs_app(incarnations[i],
+						    IPPROTO_TCP,
+						    ports[i]))) {
 				return j;
 			}
 			IP_VS_DBG(1-debug,
-                                  "Ftp: loaded support on port[%d] = %d\n",
-                                  i, ports[i]);
+				  "Ftp: loaded support on port[%d] = %d\n",
+				  i, ports[i]);
 		} else {
 			/* To be safe, force the incarnation table entry
-                           to be NULL */
+			   to be NULL */
 			incarnations[i] = NULL;
 		}
 	}
@@ -389,7 +389,7 @@ static int __init ip_vs_ftp_init(void)
 
 
 /*
- * 	ip_vs_ftp finish.
+ *	ip_vs_ftp finish.
  */
 static void __exit ip_vs_ftp_exit(void)
 {
@@ -404,7 +404,7 @@ static void __exit ip_vs_ftp_exit(void)
 				kfree(incarnations[i]);
 				incarnations[i] = NULL;
 				IP_VS_DBG(1-debug, "Ftp: unloaded support on port[%d] = %d\n",
-                                          i, ports[i]);
+					  i, ports[i]);
 			}
 		}
 	}
